#include <stdio.h>
#include <getopt.h>

#include "base.h"
#include "tree.h"
#include "proc.h"
#include "ncps.h"
#include "gui.h"


int main(int argc, char *argv[])
{
    int check = 1;
    int tmp;
    killer_t *chow;
    killer_t *hold;
    int c;
    int opt_a = 0, opt_t = 0, opt_u = 0;
    char user[20];

    signal(SIGILL, cleanup);
    signal(SIGTERM, cleanup);
    signal(SIGSEGV, cleanup);
    signal(SIGABRT, cleanup);
    signal(SIGHUP, cleanup);

    /*  printf("initscr ended\n"); */

    killer.uid = getuid();

    for (;;) {
	static struct option long_options[] =
	{
	    {"all", 0, 0, 'a'},
	    {"user", 1, 0, 'u'},
	    {"tree", 0, 0, 't'},
	    {"help", 0, 0, 'h'},
	    {0, 0, 0, 0}
	};

	c = getopt_long(argc, argv, "atu:h", long_options, NULL);
	if (c == -1)
	    break;

	switch (c) {
	case 'a':
	    opt_a = 1;
	    break;

	case 't':
	    opt_t = 1;
	    break;

	case 'u':
	    opt_u = 1;
	    strncpy(user, optarg, 19);
	    break;

	case 'h':
	    printf("ncps: Screen oriented `ps'. Version %u.%u%s\n",
		   V_MAJOR, V_MINOR, V_PATCH);
	    usage();
	    exit(0);

	case '?':
	    usage();
	    exit(1);

	default:
	    printf("?? getopt returned character code 0%o ??\n", c);
	    exit(1);
	}
    }

    /* both -a and -u cannot be present */
    if (opt_a && opt_u) {
	fprintf(stderr, "ncps: Options 'all' and 'user' are mutually "
		"incompatible.\n");
	exit(1);
    }
    if (opt_a)
	killer.uid = -1;
    else if (opt_u) {
	struct passwd *p = getpwnam(user);
	if (p == NULL) {
	    fprintf(stderr, "ncps: No such user: %s\n", user);
	    exit(1);
	} else {
	    killer.uid = p->pw_uid;
	}
    } else {
	killer.uid = getuid();	/* default: current user */
    }


    t_view = opt_t;
	

#if !DEBUG
    initscr();
    curs_set(0);
    keypad(stdscr, TRUE);
    cbreak();
    noecho();

    start_color();

    init_pair(1, COLOR_WHITE, COLOR_BLACK);
    init_pair(2, COLOR_GREEN, COLOR_BLACK);
    init_pair(3, COLOR_GREEN, COLOR_RED);
    init_pair(4, COLOR_YELLOW, COLOR_BLUE);
    init_pair(5, COLOR_BLACK, COLOR_WHITE);
    init_pair(6, COLOR_BLACK, COLOR_GREEN);
    init_pair(7, COLOR_BLACK, COLOR_CYAN);

    /* attrset(COLOR_PAIR(1)); */
    /*   print_tree(ptree, killer.pidlist, roots);   */
    /*   tmp = getch(); */

    /*gennlist(); */
#endif

    initplist(killer.uid);

    /*  printf("initplist ended\n"); */

#if !DEBUG
    clear_all_screen();
#endif

    chow = &killer;
    while (check) {
	/* printf("in main while loop\n"); */
	tmp = getchoice(chow);
	if (tmp == -1)
	    check = 0;
	else if (tmp == -2) {
	    endwin();
	    free_list(chow->pidlist, chow->count);
	    free_list(chow->trunk, chow->count);
	    initplist(killer.uid);
	    chow = &killer;
	    initscr();
	    curs_set(0);
	    clear_all_screen();
	} else if (tmp == -4) {
	    chow = chow->son;
	} else if (tmp == -5) {
	    if (chow->mother) {
		free_list(chow->pidlist, chow->count);
		free_list(chow->trunk, chow->count);
		chow = chow->mother;
		free(chow->son);

	    }
	} else if (tmp == -6) {

	    clear_all_screen();
	    endwin();
	    monitor(chow);
	    chow = chow->son;
	    selective_spew(chow);
	    /*  if(chow->count) {  */
/*              printf("%i %s\n", chow->count, chow->pidlist[0].pname); */
/*              getch(); */
/*          } */
	    initscr();
	    curs_set(0);
	    clear_all_screen();
	} else if (tmp == -7) {
	    endwin();

	    hold = chow;
	    if (hold->count)
		selective_spew(hold);

	    while (hold->mother) {
		hold = hold->mother;
		if (hold->count)
		    selective_spew(hold);
	    }

	    /*  if(chow->count) {  */
/*              printf("%i %s\n", chow->count, chow->pidlist[0].pname); */
/*              getch(); */
/*          } */
	    initscr();
	    curs_set(0);
	    clear_all_screen();
	}
    }

    clear_all_screen();
    endwin();

    return 1;
}				/* main */


void cleanup1(int nil)
{

    /*  if(chow.pidlist) */
/*      free_list(chow.pidlist); */
/*      if(chow.trunk) */
/*      free_list(chow.trunk); */

    clear_all_screen();
    endwin();

    exit(0);
}

void cleanup(int nil)
{
    clear_all_screen();
    endwin();
    fprintf(stderr, "Terminated on signal %i.\n", nil);
    exit(0);
}


void usage(void)
{
    printf("Usage: ncps [options]\n"
	   "\t-a, --all                : display all processes\n"
           "\t-u <user>, --user <user> : display processes owned by <user>\n"
	   "\t                           (defaults to the current user)\n"
	   "\t-t --tree                : starts in tree mode "
	   "(not yet implemented)\n"
	   "\t-h, --help               : display this help.\n");
}
