#include <stdio.h>
#include <regex.h>

#include "base.h"
#include "tree.h"
#include "escape.h"
#include "proc.h"

/*  unsigned read_total_main(void) { */
/*      unsigned** mem; */
/*      return (mem = meminfo()) ? mem[meminfo_main][meminfo_total] : -1; */
/*  } */

static unsigned long page_shift;
killer_t killer;


void fill_pcpu(proc_t * buf)
{
    unsigned long total_time;
    unsigned long pcpu = 0;
    unsigned long seconds;

    if (buf == NULL)
	return;			/* Just in case */
    total_time = buf->utime + buf->stime;
    total_time += (buf->cutime + buf->cstime);
    seconds = (uptime(0, 0) - ((unsigned long) buf->start_time) / Hertz);
    if (seconds)
	pcpu = ((long) total_time * 1000 / Hertz) / seconds;
    buf->pcpu = (pcpu > 999) ? 999 : pcpu;
}


static procinfo_t *simple_spew(procinfo_t * tmp, int uid)
{
    proc_t buf;
    PROCTAB *ptp;
    //int check = 1;
    int i;
    int proc_table_size = PT_SIZE;
    procinfo_t *tmptmp;

    proc_table_size = PT_SIZE;
    tmp = malloc(proc_table_size * sizeof(procinfo_t));
    if (tmp == NULL) {
	/* XXX we should realy do something about this... */
	exit(-1);
    }
    ptp = openproc(PROC_FILLCMD | PROC_FILLENV | PROC_FILLUSR | PROC_FILLBUG);

    i = 0;
    memset(&buf, '#', sizeof(proc_t));
    while (ps_readproc(ptp, &buf)) {
	if (buf.suid == uid || uid == -1) {
	    if (i > proc_table_size - 1) {	/* Pajtek */
		proc_table_size += PT_SIZE;
		tmptmp = realloc(tmp, proc_table_size * sizeof(procinfo_t));
		if (tmptmp == NULL) {
		    /* XXX some problems reallocating... */
		    exit(-1);
		}
		tmp = tmptmp;
	    }
	    tmp[i].proc = (proc_t *) malloc(sizeof(proc_t));
	    *(tmp[i].proc) = buf;	/* Pajtek */
#if DEBUG
	    printf("%i\n", tmp[i].proc->pid);
#endif
	    tmp[i].pmem = tmp[i].proc->rss * 1000 / PAGEDIV;
	    dev_to_tty(tmp[i].ttyc, TTYC, tmp[i].proc->tty,
		       tmp[i].proc->pid, ABBREV_DEV);
	    //if (tmp[i].ttyc[0] == '?')
	    //    strcpy(tmp[i].ttyc, "N/A");
	    /* tmp[i].expand = 0; */
	    /* in tree view, PID 1 is implicitely expanded */
	    tmp[i].expand = (tmp[i].proc->pid == 1) ? 1 : 0;
	    tmp[i].is_selected = 0;
	    fill_pcpu(tmp[i].proc);
	    get_lc(&(tmp[i]));
	    tmp[i].s_pname = strdup(tmp[i].pname);
#if DEBUG
	    printf("%s\n", tmp[i].pname);
#endif
	    i++;
	    memset(&buf, '#', sizeof(proc_t));
	}
    }

    killer.count = i;

#if DEBUG
    for (i = 0; i < killer.count; i++) {
	printf("%i\n", tmp[i].proc->pid);
    }
#endif

#if DEBUG
    printf("loop over\n");
#endif

    closeproc(ptp);
    if (killer.count)
	return tmp;
    else
	return NULL;
}

void initplist(int view)
{
#if DEBUG
    int i;
#endif
    procinfo_t *tmp;		//, *hol;

#if DEBUG
    printf("initplist\n");
#endif

    meminfo();
    switch (getpagesize()) {
    case 65536:
	page_shift = 16;
	break;
    case 32768:
	page_shift = 15;
	break;
    case 16384:
	page_shift = 14;
	break;
    case 8192:
	page_shift = 13;
	break;
    default:
	fprintf(stderr, "Unknown page size! (assume 4096)\n");
    case 4096:
	page_shift = 12;
	break;
    case 2048:
	page_shift = 11;
	break;
    case 1024:
	page_shift = 10;
	break;
    }

    killer.monitoring = 0;
    killer.mother = NULL;

    killer.pidlist = simple_spew(tmp, view);

#if DEBUG
    printf("spew complete\n");
#endif

    killer.trunk = (procinfo_t *) calloc(killer.count, sizeof(procinfo_t));

#if DEBUG
    printf("killer.count %i\n", killer.count);
    for (i = 0; i < killer.count; i++) {
	printf("%s\n", killer.pidlist[i].pname);
    }
#endif
}


void search(killer_t * mother, char *input)
{
    int i;
    regex_t preg;

    mother->son = (killer_t *) malloc(sizeof(killer_t));
    mother->son->pidlist = (procinfo_t *) calloc(mother->count, sizeof(procinfo_t));
    mother->son->trunk = (procinfo_t *) calloc(mother->count, sizeof(procinfo_t));
    mother->son->monitoring = 0;

    mother->son->mother = mother;

    mother->son->count = 0;

    if (regcomp(&preg, input, REG_ICASE | REG_NOSUB | REG_EXTENDED) != 0)
	return;

    for (i = 0; i < mother->count; i++) {
	if (regexec(&preg, mother->pidlist[i].pname, 0, NULL, 0) == 0) {
	    memcpy(&(mother->son->pidlist[mother->son->count]),
		   &(mother->pidlist[i]), sizeof(procinfo_t));
	    mother->son->pidlist[mother->son->count].proc = (proc_t *) malloc(sizeof(proc_t));
	    memset(mother->son->pidlist[mother->son->count].proc,
		   '#', sizeof(proc_t));
	    memcpy(mother->son->pidlist[mother->son->count].proc,
		   mother->pidlist[i].proc, sizeof(proc_t));
	    mother->son->pidlist[mother->son->count].pname = strdup(mother->pidlist[i].pname);
	    mother->son->pidlist[mother->son->count].s_pname = strdup(mother->pidlist[i].s_pname);

	    /*  endwin(); */
/*          printf("%i\n",  mother->son->pidlist[mother->son->count].proc->pid); */
	    mother->son->count++;
	}
    }
}


void get_lc(procinfo_t * input)
{
    char tag[256], *line = tag;
    const char **lc = (const char **) input->proc->cmdline;

    if (lc && *lc) {
	escape_strlist(line, lc, 256);
    } else {
	/* cmd between '[' and ']' */
	*line = '[';
	escape_str(line + 1, input->proc->cmd, 250);
	strcat(line, "]");
    }
    input->pname = strdup(tag);
}


void monitor(killer_t * mother)
{
    int i;

    mother->son = (killer_t *) malloc(sizeof(killer_t));

    mother->son->pidlist = (procinfo_t *) calloc(mother->count, sizeof(procinfo_t));
    mother->son->trunk = (procinfo_t *) calloc(mother->count, sizeof(procinfo_t));

    mother->son->monitoring = 1;
    mother->son->mother = mother;

    mother->son->count = 0;
    mother->son->t_count = 0;

    for (i = 0; i < mother->count; i++) {
	if (mother->pidlist[i].is_selected) {
	    dup_procinfo_t(&(mother->son->pidlist[mother->son->count]), &(mother->pidlist[i]));
	    mother->son->pidlist[mother->son->count].s_pname = strdup(mother->son->pidlist[mother->son->count].pname);
	    mother->son->count++;
	}
    }
}


void selective_spew(killer_t * chow)
{
    procinfo_t *tmp;		//, *hold;

    proc_t buf;
    pid_t *list_buf;
    PROCTAB *ptp;
    int i;
    int j;
    int check;
    int proc_table_size = PT_SIZE;

    list_buf = (pid_t *) calloc(chow->count, sizeof(pid_t));

    for (i = 0; i < chow->count; i++) {
	list_buf[i] = chow->pidlist[i].proc->pid;
#if DEBUG
	printf("%i\n", list_buf[i]);
#endif
    }

    proc_table_size += PT_SIZE;
    memset(&buf, '#', sizeof(proc_t));
    tmp = malloc(proc_table_size * sizeof(procinfo_t));

    ptp = openproc(PROC_FILLCMD | PROC_FILLENV | PROC_FILLUSR | PROC_FILLBUG);

    i = 0;
    while (ps_readproc(ptp, &buf) && i < chow->count) {
	j = 0;
	check = 1;
	while (j < chow->count && check) {
	    if (buf.pid == list_buf[j]) {
#if DEBUG
		printf("%i", buf.pid);
#endif
		if (i > proc_table_size - 1) {	/* Pajtek */
		    proc_table_size += PT_SIZE;
		    tmp = realloc(tmp, proc_table_size * sizeof(procinfo_t));
		}
		tmp[i].proc = (proc_t *) malloc(sizeof(proc_t));
		*(tmp[i].proc) = buf;	/* Pajtek */
#if DEBUG
		printf("%i\n", tmp[i].proc->pid);
#endif
		tmp[i].pmem = tmp[i].proc->rss * 1000 / PAGEDIV;
		dev_to_tty(tmp[i].ttyc, TTYC, tmp[i].proc->tty,
			   tmp[i].proc->pid, ABBREV_DEV);
		tmp[i].expand = 0;
		tmp[i].is_selected = 0;
		fill_pcpu(tmp[i].proc);
		get_lc(&(tmp[i]));
		tmp[i].s_pname = strdup(tmp[i].pname);
#if DEBUG
		printf("%s\n", tmp[i].pname);
#endif
		check = 0;
		i++;
	    } else
		j++;
	}
	memset(&buf, '#', sizeof(proc_t));
    }

    closeproc(ptp);

    free(list_buf);

    chow->pidlist = tmp;
    chow->trunk = (procinfo_t *) calloc(i, sizeof(procinfo_t));

    chow->count = i;
    chow->t_count = 0;

    chow->trunk = (procinfo_t *) calloc(chow->count, sizeof(procinfo_t));
}
